﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAircraftCarrier
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    internal class DrawingAircraftCarrier
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityAircraftCarrier? EntityAircraftCarrier { get; private set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// Левая координата прорисовки авианосца
        /// </summary>
        private int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки авианосца
        /// </summary>
        private int _startPosY;
        /// <summary>
        /// Ширина прорисовки авианосца
        /// </summary>
        private readonly int _carrierWidth = 196;
        /// <summary>
        /// Высота прорисовки авианосца
        /// </summary>
        private readonly int _carrierHeight = 58;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="cabin">Признак наличия кабины</param>
        /// <param name="runway">Признак наличия ввп</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена,нельзя создать объект в этих размерах</returns>
        public bool Init(int speed, double weight, Color bodyColor, Color
additionalColor, bool cabin, bool runway, int width, int height)
        {
            if (width < _carrierWidth || height < _carrierHeight)
                return false;
            _pictureWidth = width;
            _pictureHeight = height;
            EntityAircraftCarrier = new EntityAircraftCarrier();
            EntityAircraftCarrier.Init(speed, weight, bodyColor, additionalColor, cabin, runway);
            return true;
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if (x < 0 || y < 0 || x + _carrierWidth > _pictureWidth || y + _carrierHeight > _pictureHeight)
            {
                x = 0;
                y = 0;
            }
            _startPosX = x;
            _startPosY = y;
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (EntityAircraftCarrier == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    if (_startPosX - EntityAircraftCarrier.Step > 0)
                    {
                        _startPosX -= (int)EntityAircraftCarrier.Step;
                    }
                    break;
                //вверх
                case DirectionType.Up:
                    if (_startPosY - EntityAircraftCarrier.Step > 0)
                    {
                        _startPosY -= (int)EntityAircraftCarrier.Step;
                    }
                    break;
                // вправо
                case DirectionType.Right:
                    if (_startPosX + _carrierWidth + EntityAircraftCarrier.Step < _pictureWidth)
                    {
                        _startPosX += (int)EntityAircraftCarrier.Step;
                    }
                    break;
                //вниз
                case DirectionType.Down:
                    if (_startPosY + _carrierHeight + EntityAircraftCarrier.Step < _pictureHeight)
                    {
                        _startPosY += (int)EntityAircraftCarrier.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (EntityAircraftCarrier == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush additionalBrush = new SolidBrush(EntityAircraftCarrier.AdditionalColor);
            Brush bodyBrush = new SolidBrush(EntityAircraftCarrier.BodyColor);
            //двигатели
            Point[] PointEngine1 = {
                new Point(_startPosX + 0, _startPosY + 11),
                new Point(_startPosX + 2, _startPosY + 11),
                new Point(_startPosX + 2, _startPosY + 24),
                new Point(_startPosX + 0, _startPosY + 24),
            };
            Point[] PointEngine2 = {
                new Point(_startPosX + 0, _startPosY + 31),
                new Point(_startPosX + 2, _startPosY + 31),
                new Point(_startPosX + 2, _startPosY + 44),
                new Point(_startPosX + 0, _startPosY + 44),
            };
            Brush brBlack = new SolidBrush(Color.Black);
            g.FillPolygon(brBlack, PointEngine1);
            g.DrawPolygon(pen, PointEngine1);
            g.FillPolygon(brBlack, PointEngine2);
            g.DrawPolygon(pen, PointEngine2);
            //границы авианосца
            Point[] PointBorders = {
                new Point(_startPosX + 4, _startPosY + 0),
                new Point(_startPosX + 142, _startPosY + 0),
                new Point(_startPosX + 196, _startPosY + 29),
                new Point(_startPosX + 142, _startPosY + 58),
                new Point(_startPosX + 4, _startPosY + 58),
            };
            g.FillPolygon(bodyBrush, PointBorders);
            g.DrawPolygon(pen, PointBorders);
            //отсеки
            g.DrawRectangle(pen, _startPosX + 98, _startPosY + 18, 10, 23);
            g.FillRectangle(additionalBrush, _startPosX + 99, _startPosY + 19, 9, 22);
            g.DrawRectangle(pen, _startPosX + 78, _startPosY + 23, 20, 13);
            g.FillRectangle(additionalBrush, _startPosX + 79, _startPosY + 24, 19, 12);
            g.DrawEllipse(pen, _startPosX + 138, _startPosY + 22, 11, 11);
            g.FillEllipse(additionalBrush, _startPosX + 138, _startPosY + 22, 11, 11);
            //впп
            if (EntityAircraftCarrier.Runway)
            {
                Pen penWhite = new Pen(Color.White, 3);
                Pen penThinWhite = new Pen(Color.White, 1);
                g.DrawLine(penWhite, _startPosX + 4, _startPosY + 14, _startPosX + 114, _startPosY + 1);
                g.DrawLine(penWhite, _startPosX + 4, _startPosY + 26, _startPosX + 156, _startPosY + 8);
                g.DrawLine(penThinWhite, _startPosX + 3, _startPosY + 20, _startPosX + 148, _startPosY + 3);
            }
            // кабина
            if (EntityAircraftCarrier.Cabin)
            {
                Point[] PointCabin = {
                new Point(_startPosX + 15, _startPosY + 42),
                new Point(_startPosX + 17, _startPosY + 41),
                new Point(_startPosX + 22, _startPosY + 41),
                new Point(_startPosX + 22, _startPosY + 38),
                new Point(_startPosX + 25, _startPosY + 38),
                new Point(_startPosX + 25, _startPosY + 41),
                new Point(_startPosX + 35, _startPosY + 41),
                new Point(_startPosX + 35, _startPosY + 38),
                new Point(_startPosX + 38, _startPosY + 38),
                new Point(_startPosX + 38, _startPosY + 41),
                new Point(_startPosX + 43, _startPosY + 41),
                new Point(_startPosX + 43, _startPosY + 38),
                new Point(_startPosX + 46, _startPosY + 38),
                new Point(_startPosX + 46, _startPosY + 41),
                new Point(_startPosX + 51, _startPosY + 41),
                new Point(_startPosX + 52, _startPosY + 42),
                new Point(_startPosX + 52, _startPosY + 45),
                new Point(_startPosX + 51, _startPosY + 46),
                new Point(_startPosX + 46, _startPosY + 46),
                new Point(_startPosX + 46, _startPosY + 49),
                new Point(_startPosX + 43, _startPosY + 49),
                new Point(_startPosX + 43, _startPosY + 46),
                new Point(_startPosX + 38, _startPosY + 46),
                new Point(_startPosX + 38, _startPosY + 49),
                new Point(_startPosX + 35, _startPosY + 49),
                new Point(_startPosX + 35, _startPosY + 46),
                new Point(_startPosX + 25, _startPosY + 46),
                new Point(_startPosX + 25, _startPosY + 49),
                new Point(_startPosX + 22, _startPosY + 49),
                new Point(_startPosX + 22, _startPosY + 46),
                new Point(_startPosX + 17, _startPosY + 46),
                new Point(_startPosX + 15, _startPosY + 45),
                };
                g.FillPolygon(additionalBrush, PointCabin);
                g.DrawPolygon(pen, PointCabin);
            }
        }
    }
}
